/**
 * Javier Abelln, 31 Mayo 2006
 */
package chuidiang.ejemplos.base_datos;

import java.sql.ResultSet;
import java.sql.ResultSetMetaData;

import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableModel;

/**
 * Convierte un ResultSet en un DefaultTableModel
 * @author Chuidiang
 *
 */
public class ConversorResultSetADefaultTableModel
{
    /**
     * Rellena el DefaultTableModel con los datos del ResultSet.
     * Vaca el DefaultTableModel completamente y le mete los datos que hay
     * en el ResultSet.
     * @param rs El resultado de lac onsula a base de datos.
     * @param modelo El DefaultTableModel que queremos rellenar
     */
    public static void rellena(ResultSet rs, DefaultTableModel modelo)
    {
        configuraColumnas(rs, modelo);
        vaciaFilasModelo(modelo);
        anhadeFilasDeDatos(rs, modelo);
    }

    /**
     * Aade al DefaultTableModel las filas correspondientes al ResultSet.
     * @param rs El resultado de la consulta a base de datos
     * @param modelo El DefaultTableModel que queremos rellenar.
     */
    private static void anhadeFilasDeDatos(ResultSet rs,
            DefaultTableModel modelo)
    {
        int numeroFila = 0;
        try
        {
            // Para cada registro de resultado en la consulta 
            while (rs.next())
            {
                // Se crea y rellena la fila para el modelo de la tabla.
                Object[] datosFila = new Object[modelo.getColumnCount()];
                for (int i = 0; i < modelo.getColumnCount(); i++)
                    datosFila[i] = rs.getObject(i + 1);
                modelo.addRow(datosFila);
                numeroFila++;
            }
            rs.close();
        } catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Borra todas las filas del modelo.
     * @param modelo El modelo para la tabla.
     */
    private static void vaciaFilasModelo(final DefaultTableModel modelo)
    {
        // La llamada se hace in un invokeAndWait para que se ejecute en el
        // hilo de refresco de ventanas y evitar que salten excepciones
        // durante dicho refresco.
        try
        {
            SwingUtilities.invokeAndWait(new Runnable()
            {

                public void run()
                {
                    while (modelo.getRowCount() > 0)
                        modelo.removeRow(0);
                }

            });
        } catch (Exception e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Pone en el modelo para la tabla tantas columnas como tiene el resultado
     * de la consulta a base de datos.
     * @param rs Resultado de consulta a base de datos.
     * @param modelo Modelo de la tabla.
     */
    public static void configuraColumnas(final ResultSet rs,
            final DefaultTableModel modelo)
    {
        try
        {
            // Se hace en un invokeAndWait para que este cdigo se ejecute
            // en el hilo de refresco de ventanas, evitando que salten
            // excepciones.
            SwingUtilities.invokeAndWait(new Runnable()
            {

                public void run()
                {
                    try
                    {
                        // Se obtiene los metadatos de la consulta. Con ellos
                        // podemos obtener el nmero de columnas y el nombre
                        // de las mismas.
                        ResultSetMetaData metaDatos = rs.getMetaData();
                        
                        // Se obtiene el numero de columnas.
                        int numeroColumnas = metaDatos.getColumnCount();
                        
                        // Se obtienen las etiquetas para cada columna
                        Object[] etiquetas = new Object[numeroColumnas];
                        for (int i = 0; i < numeroColumnas; i++)
                        {
                            etiquetas[i] = metaDatos.getColumnLabel(i + 1);
                        }
                        
                        // Se meten las etiquetas en el modelo. El numero
                        // de columnas se ajusta automticamente.
                        modelo.setColumnIdentifiers(etiquetas);
                    } catch (Exception e)
                    {
                        e.printStackTrace();
                    }

                }

            });
        } catch (Exception e)
        {
            e.printStackTrace();
        }
    }
}
